# Functions commonly used by scripted-partitioning scripts

# Log a message.
log() {
	logger -t scripted-partitioning "$@"
}

# Log a fatal error and quit.
fatal() {
	logger -t scripted-partitioning -p user.err "$@"
	exit 1
}

prep_system() {
	depmod -a || true

	log "dm-mod"
	really_load_module dm_mod

	really_load_module ext2

	# Make sure LVM is installed into /target/ as well
	apt-install lvm2 || true

	log "Creating /target"
	mkdir -p /target || true
}

# Generate a device path (/dev/[sh]d[a-h][1-9], typically) for given disc
# and partition numbers.  If you leave off the partition number, you'll get
# the entire disk instead (/dev/[sh]d[a-h]).
#
# For maximum confusion, the disk number is zero-based, counting from the
# first disk, whilst the partition number is the partition number as per the
# standard conventions for Linux-based partitioning (primaries 1-4,
# secondaries 5+).
#
# Usage: disk_part <disk number> [partition number]
#
disk_part() {
	[ -n "$1" ] || fatal "disk_part called without a disk number!"

	# Special case: if the 'disk number' has a slash in it, then
	# someone's passed us a full path instead...
	if echo "$1" | grep -q /; then
		echo "$1"
		return
	fi

	local part=""
	
	if [ -n "$2" ]; then
		local disk="$(disk_part $1)"
		# This odd regex is designed to catch both the simple
		# /dev/sda1 case, as well as HP's insane /dev/cciss/c0d0p1
		# scheme.
		part="$(list-devices partition | grep "^${disk}[a-z]*${2}\$")"
	else
		part="$(list-devices disk | sed -n $(($1 + 1))p)"
	fi
	if [ -z "$part" ]; then
		log "disk_part $1 $2 came up with nothing"
	fi
	
	log "disk_part '$1' '$2' maps to '$part'"
	echo $part
}

# Loop until we get a block device corresponding to the relevant disk and
# partition that we specify.
#
# Usage: wait_for_disk_partition <disk number> <partition number>
#
wait_for_disk_partition() {
	log "Going to wait for 'disk_part $1 $2'"

	local PART="$(disk_part $1 $2)"
	
	while [ -z "$PART" ]; do
		log "Still waiting (list-devices doesn't show it)..."
		sleep 1
		PART="$(disk_part $1 $2)"
	done

	while [ ! -b "$PART" ]; do
		log "Still waiting (not a block device)..."
		sleep 1
	done

	log "Got it!"
}

# Loop until the block device specified by the given disk and partition
# number is no longer available.
#
# Usage: wait_until_disk_partition_is_gone <disk number> <partition number>
#
wait_until_disk_partition_is_gone() {
	PART="$(disk_part $1 $2)"
	
	if [ -z "$PART" ]; then
		log "disk_part $1 $2 is already gone"
	else
		log "Going to wait for '${PART}' to go away"
	
		while [ -b "$PART" ]; do
			log "Still waiting..."
			sleep 1
		done
	fi

	log "disk_part $1 $2 is gone now!"
}

really_load_module() {
	local mod="$1"
	
	if ! grep -q $mod /proc/modules; then
		log "Going to load $mod"
	
		modprobe $mod
	
		sleep 2
		while ! grep -q $mod /proc/modules; do
			log "Not available yet, going to try again..."
			modprobe $mod
			sleep 2
		done
	
		log "Got it!"
	fi
}

# Partition disk in a standard way for boot / LVM.  Pass the disk number in
# (0-indexed, of course) that you want to work on.  Note that this will
# completely wipe the existing contents of the disk.
#
partition_disk_for_boot_and_lvm() {
	local DISK_NUMBER="$1"
	local DISK="$(disk_part $DISK_NUMBER)"
	
	# The disk needs to exist!
	if [ ! -b ${DISK} ]; then
		fatal "The disk number you specified ($1) does not exist."
	fi
	
	log "parted"

	# First off, do we need a disklabel?
	if ! /sbin/parted --script $DISK print >/dev/null; then
		log "We need a disklabel"
		/sbin/parted --script $DISK "mklabel msdos"
		log "Disklabel created"
	else
		log "Disklabel already exists; no action required"
	fi

	# Delete any existing partitions (yes, this is dangerous)
	# The reverse sort is to make sure we delete extended partitions
	# before we delete the primary partition they're contained in.
	basedisk=$(echo $DISK | sed 's%^/dev/%%')
	
	log "Deleting partitions from $basedisk"

	for minor in $(grep $basedisk\. /proc/partitions | sed 's/^.*\([0-9][0-9]*\)$/\1/' | sort -r); do
		log "Removing $minor"
		/sbin/parted --script $DISK "rm $minor"
		wait_until_disk_partition_is_gone $DISK_NUMBER $minor
	done

	log "Partitions deleted"

	# Now let's go wild!
	/sbin/parted --script ${DISK} "mkpart primary 0 200"
	wait_for_disk_partition $DISK_NUMBER 1
	/sbin/parted --script ${DISK} "mkpart primary 200 -0"
	wait_for_disk_partition $DISK_NUMBER 2
	/sbin/parted --script ${DISK} "set 2 lvm on"
}

# Create an LVM volume group named for the host on the physical volume(s)
# specified.
#
# Usage: lvm_setup <physical volume> [<physical volume> ...]
#
lvm_setup() {
	log "pvcreate"
	for lvmpart in "$@"; do
		pvcreate -ff -y "${lvmpart}"
	done
	sleep 1
	log "vgcreate"
	vgcreate $(cat /etc/hostname) "$@"
}

# Create a logical volume in the volume group named for the host, with the
# given size and mount point, make a filesystem on it, mount it in the
# appropriate location, and write an entry in the target fstab for the LV.
#
# A couple of caveats on the use of this method:
#
# * Do not create a separate mount point for /etc (who would, anyway?)
#   because the fstab creation will get *very* confused;
#
# * Do not create an LV with a name containing a hyphen, because we don't
#   escape the hyphens properly yet.
#
# Usage: create_lv <LV name> <size> <mount point> <fstype>
#
# <size> should be specified in "lvcreate" form, eg "2G".  We treat a
# mountpoint of "swap" specially, and use that as swap space instead of
# trying to mount it.
#
create_lv() {
	local LVNAME="$1"
	local LVSIZE="$2"
	local MOUNTPOINT="$3"
	local FSTYPE="$4"
	local VG="$(cat /etc/hostname)"
	
	# Backwards compatibility ftw
	if [ -z "$FSTYPE" ]; then
		if [ "$MOUNTPOINT" = "swap" ]; then
			FSTYPE=swap
		else
			FSTYPE=xfs
		fi
	fi
		
	[ -n "$LVNAME" ] || fatal "No LV name given"
	log "Creating LV $LVNAME"
	
	[ -n "$LVSIZE" ] || fatal "No LV size given"
	[ -n "$MOUNTPOINT" ] || fatal "No mount point specified"
	
	log "lvcreate -L $LVSIZE -n $LVNAME $VG"
	lvcreate -L "$LVSIZE" -n "$LVNAME" $VG

	wait_for_disk_partition "/dev/$VG/$LVNAME"
	
	if [ "$MOUNTPOINT" = "swap" ]; then
		log "Making swap on /dev/$VG/$LVNAME"
		
		mkswap "/dev/$VG/$LVNAME"
		swapon "/dev/$VG/$LVNAME"
	else
		# Making a bit of an assumption here...
		really_load_module "$FSTYPE"
		
		log "Creating $FSTYPE filesystem on /dev/$VG/$LVNAME"
		mkfs.$FSTYPE "/dev/$VG/$LVNAME"

		log "Creating mount point for $LVNAME"
		mkdir -p "/target${MOUNTPOINT}"
	
		mount "/dev/$VG/$LVNAME" "target${MOUNTPOINT}"
	fi
	
	# Write out an fstab entry for the LV
	[ -d /target/etc ] || mkdir -p /target/etc

	echo "/dev/mapper/$VG-${LVNAME} ${MOUNTPOINT} ${FSTYPE} defaults 0 0" >>/target/etc/fstab
}

# Setup the /boot partition on the given block device.
#
# Usage: setup_boot <partition>
#
setup_boot() {
	BOOTPART="$1"

	log "Creating /target/boot"
	mkdir /target/boot

	wait_for_disk_partition ${BOOTPART}

	log "Making ext2 on /boot"
	mkfs.ext2 ${BOOTPART}

	while [ ! -d /target/boot/lost+found ]; do
		log "Mounting /boot"
		mount -t ext2 ${BOOTPART} /target/boot || true
	done

	log "Adding /boot to fstab"
	echo "$BOOTPART /boot ext2 defaults,errors=remount-ro 0 2" >>/target/etc/fstab
}

# Setup a tmpfs-backed /tmp
setup_tmp() {
	log "Making /target/tmp"
	mkdir /target/tmp

	log "Mounting /tmp"
	mount -t tmpfs tmpfs /target/tmp -o mode=1777

	echo "tmpfs /tmp tmpfs defaults,mode=1777 0 0" >>/target/etc/fstab
}

# Just write out an initial fstab in /target, with the standard stuff that
# every fstab has.
#
write_static_fstab_bits() {
	local CDROMDEV="$(list-devices cd | head -n 1)"
	
	mkdir /target/proc
	mkdir /target/cdrom
	
	cat <<EOF >>/target/etc/fstab
proc			/proc	proc	defaults			0 0
EOF
	if [ -n "$CDROMDEV" ]; then
		cat <<EOF >>/target/etc/fstab
$CDROMDEV	/cdrom	iso9660,udf ro,user,noauto		0 0
EOF
	fi
}
